package com.zy.bms.config.mqtt;

import com.zy.bms.handler.MqttCallbackHandler;
import com.zy.bms.utils.RandomCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.core.MessageProducer;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.inbound.MqttPahoMessageDrivenChannelAdapter;
import org.springframework.integration.mqtt.support.DefaultPahoMessageConverter;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;

import java.util.Objects;

/**
 * MQTT 服务 消费者配置
 *
 * @author yang xiao kun
 * create on 2021/1/19
 */
@Configuration
public class MqttConsumerCfg {

    @Autowired
    private MqttCallbackHandler mqttCallbackHandler;

    @Autowired
    private MqttPahoClientFactory mqttClientFactory;

    //默认监听主题
    private final String[] defaultTopic = new String[]{"$regdtx2", "$dtx2/#"};

    /**
     * MQTT 消息订阅绑定（消费者）
     * 可以同时消费（订阅）多个Topic
     */
    @Bean
    public MessageProducer inbound() {
        // 初始化入站通道适配器，使用的是Eclipse Paho MQTT客户端库
        MqttPahoMessageDrivenChannelAdapter adapter =
                new MqttPahoMessageDrivenChannelAdapter(RandomCode.UUID_8(), mqttClientFactory, defaultTopic);
        // 设置连接超时时长 毫秒
        adapter.setCompletionTimeout(5000);
        // 配置默认Paho消息转换器(qos=0, retain=false, charset=UTF-8)
        adapter.setConverter(new DefaultPahoMessageConverter());
        /*
            设置服务质量
            0 最多一次，数据可能丢失;
            1 至少一次，数据可能重复;
            2 只有一次，有且只有一次;最耗性能;
         */
        adapter.setQos(0);
        // 设置订阅通道
        adapter.setOutputChannel(mqttInboundChannel());
        return adapter;
    }

    /**
     * MQTT信息通道（消费者）
     */
    @Bean(name = "mqttInboundChannel")
    public MessageChannel mqttInboundChannel() {
        return new DirectChannel();
    }

    /**
     * MQTT消息处理器（消费者）
     */
    @Bean
    @ServiceActivator(inputChannel = "mqttInboundChannel")
    public MessageHandler handler() {
        return message -> {
            String topic = Objects.requireNonNull(message.getHeaders().get("mqtt_receivedTopic")).toString();
            String payload = message.getPayload().toString();
            mqttCallbackHandler.handle(topic, payload);
        };
    }
}
